==> newland-toubiz-api-dev-release-1.13.1-090799.zip: Classes/Service/Tportal/ObjectAdapter/LodgingAdapter.php <==
<?php declare(strict_types=1);
namespace Newland\Toubiz\Api\Service\Tportal\ObjectAdapter;

/*
 * This file is part of the "toubiz-api" package.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */

use Newland\Toubiz\Api\ObjectAdapter\AddressAdapterInterface;
use Newland\Toubiz\Api\ObjectAdapter\ArticleAdapterInterface;
use Newland\Toubiz\Api\ObjectAdapter\Concern\ArticleConstants;
use Newland\Toubiz\Api\ObjectAdapter\Attributes\LodgingAttributes;
use Newland\Toubiz\Api\Service\LanguageAware;
use Newland\Toubiz\Api\Service\Toubiz\Legacy\ObjectAdapter\AbstractLegacyObjectAdapter;
use Newland\Toubiz\Api\Service\UsesFirstMediumAsMainMedium;
use Newland\Toubiz\Api\Service\WithCacheProperty;
use Newland\Toubiz\Api\Utility\AttributeImportUtility;

/**
 * Lodging adapter.
 *
 * This represents an Article with mapping for the Tportal lodging.
 */
class LodgingAdapter extends AbstractLegacyObjectAdapter implements ArticleAdapterInterface
{
    use LanguageAware;
    use WithCacheProperty;
    use UsesFirstMediumAsMainMedium;

    /** @var string|null */
    private $detailUriTemplate;

    public function setDetailUriTemplate(?string $detailUriTemplate): void
    {
        $this->detailUriTemplate = $detailUriTemplate;
    }

    public function getMainType(): int
    {
        return ArticleConstants::TYPE_LODGING;
    }

    public function getExternalId(): string
    {
        return (string) $this->object['housedata']['out']['ID'];
    }

    public function getName(): string
    {
        $name = $this->object['housedata']['out']['sortName'];
        if (empty($name)) {
            $name = $this->object['housedata']['out']['address']['personFirstName'];
            $name .= ' ';
            $name .= $this->object['housedata']['out']['address']['personLastName'];
        }
        if (empty(trim($name))) {
            $name = $this->object['housedata']['out']['describingName'];
        }

        // The name may end with a comma or a dash.
        $suffix = $name[strlen($name) - 1];
        if ($suffix === ',' || $suffix === '-') {
            $name = substr($name, 0, -1);
        }

        return $name;
    }

    public function getLatitude(): ?float
    {
        return $this->object['housedata']['out']['address']['gmapsLatitude'] ?? null;
    }

    public function getLongitude(): ?float
    {
        return $this->object['housedata']['out']['address']['gmapsLongitude'] ?? null;
    }

    public function getMainAddress(): AddressAdapterInterface
    {
        $address = $this->object;
        $address['name'] = $this->getName();
        return new LodgingAddressAdapter($address);
    }

    public function getAbstract(): ?string
    {
        return null;
    }

    public function getDescription(): ?string
    {
        $part = $this->object['housedata']['out']['descriptions'];
        if ($part) {
            if (array_key_exists('description', $part)) {
                return $part['description'];
            }

            $descriptionPart = $this->findInArray(
                $part,
                'descriptionTypeName',
                'Beschreibung'
            );

            if ($descriptionPart && !empty($descriptionPart['description'])) {
                return $descriptionPart['description'];
            }
        }

        return null;
    }

    /**
     * @return CategoryAdapter[]
     */
    public function getCategories(): array
    {
        if (empty($this->object['housedata']['out']['touristicType'])) {
            return [];
        }

        return [ new CategoryAdapter($this->object['housedata']['out']['touristicType']) ];
    }

    public function getAddresses(): array
    {
        return [];
    }

    /**
     * @return MediumAdapter[]
     */
    public function getMedia(): array
    {
        return $this->cache(
            'media',
            function () {
                if (!empty($this->object['housedata']['out']['multimediaURLs'])) {
                    foreach ($this->object['housedata']['out']['multimediaURLs'] as $item) {
                        /*
                         * Right now, only the preview image is being imported as there is
                         * no show view for lodgings.
                         *
                         * Next to that, $item can also be a non-array wit useless
                         * information (like a string), or an array with an unexpected
                         * structure that does not provide a complete image.
                         */
                        if (is_array($item) && ($item['type']['value'] ?? null) === 'Preview') {
                            return [ new MediumAdapter($item) ];
                        }
                    }
                }

                /*
                 * And for the case where there is no 'TP_Preview' despite that every
                 * record basically must have one, there is 'previewImageURL' but with
                 * an already cropped image.
                 */
                $url = $this->object['searchresult']['detailInfo']['previewImageURL'] ?? '';
                if (!empty($item['url'])) {
                    $item = [
                        'ID' => md5($url),
                        'url' => $url,
                    ];
                    return [ new MediumAdapter($item) ];
                }

                return [];
            }
        );
    }

    public function hasAttributes(): bool
    {
        return !empty($this->parseAttributes());
    }

    public function getAttributes(): array
    {
        return AttributeImportUtility::splitAttributeArrayIntoImportables(
            $this->parseAttributes(),
            function ($name, $value) {
                return new AttributeAdapter($this->getExternalId(), $name, $value);
            }
        );
    }

    public function getFiles(): array
    {
        return [];
    }

    public function getSourceName(): ?string
    {
        return null;
    }

    public function getAuthorName(): ?string
    {
        return null;
    }

    public function getBookingUri(): ?string
    {
        return '';
    }

    public function getDetailUri(): ?string
    {
        if (empty($this->object['housedata']['out']['niceUrl'])) {
            return null;
        }

        // Legacy fallback for previous URL handling.
        // TODO Remove together with legacy handling in ArticleUrlService
        if (!$this->detailUriTemplate) {
            return $this->object['housedata']['out']['niceUrl'];
        }

        $niceUrl = ltrim($this->object['housedata']['out']['niceUrl'], '/');
        return str_replace('{tportalUriSegment}', $niceUrl, $this->detailUriTemplate);
    }

    public function getOpeningTimes(): ?string
    {
        return null;
    }

    public function getOpeningTimesFormat(): ?string
    {
        return null;
    }

    public function getStarClassification(): ?int
    {
        $attributes = $this->object['housedata']['out']['dataClassAttributeValues'] ?? [];
        $starAttributes = array_values(
            array_filter(
                $attributes,
                function (array $attribute) {
                    return ($attribute['twsDataClassAttribute']['name'] ?? null) === 'ClassificationServiceProvider';
                }
            )
        );

        if (!empty($starAttributes)) {
            // @todo: more than 1 star classification might be present,
            // e.g. for multiple apartments within 1 apartment complex
            $classificationString = (string) ($starAttributes[0]['twsLookupValues']['value'] ?? null);

            // Parse actual numbers
            if (preg_match('/^\d$/', $classificationString)) {
                return (int) $classificationString;
            }

            // parse the int value of rating string like "5 Sterne Superior"
            if (preg_match('/^(\d)\s.*/', $classificationString, $matches)) {
                return (int) $matches[1];
            }

            // Parse int data such as F*** as 3 stars (number of stars in the string)
            if (preg_match('/(\*+)$/', $classificationString, $matches)) {
                return \strlen($matches[1]);
            }
        }

        return null;
    }

    public function getAverageRating(): ?int
    {
        $rating = $this->object['searchresult']['serviceProviderRatingTotal']['averageGrade'] ?? null;
        if ($rating !== null) {
            return (int) $rating * 10;
        }
        return null;
    }

    public function getNumberOfRatings(): ?int
    {
        $numberOfRatings = $this->object['searchresult']['serviceProviderRatingTotal']['numberOfRatings'] ?? null;
        if ($numberOfRatings !== null) {
            return (int) $numberOfRatings;
        }
        return null;
    }

    private function parseAttributes(): array
    {
        $attributes = [];

        if (!empty($this->object['searchresult_details']['services'])) {
            $currency = $this->object['searchresult_details']['services'][0]['minimumBasePrice']['currency'];
            $unit = $this->object['searchresult_details']['services'][0]['minimumBasePriceCalculationRule'];
            $value = $this->object['searchresult_details']['services'][0]['minimumBasePrice']['value'];

            if ($currency && $unit && $value) {
                $attributes[LodgingAttributes::MINIMUM_BASE_PRICE] = [
                    LodgingAttributes::PRICE_CURRENCY => $currency,
                    LodgingAttributes::PRICE_UNIT => $unit,
                    LodgingAttributes::PRICE_VALUE => $value,
                ];
            }
        }
        return $attributes;
    }
}

==> newland-toubiz-api-dev-release-1.14.4-718839.zip: Classes/Service/Tportal/ObjectAdapter/LodgingAdapter.php <==
<?php declare(strict_types=1);
namespace Newland\Toubiz\Api\Service\Tportal\ObjectAdapter;

/*
 * This file is part of the "toubiz-api" package.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */

use Newland\Toubiz\Api\ObjectAdapter\AddressAdapterInterface;
use Newland\Toubiz\Api\ObjectAdapter\Article\ArticleWithStarRatings;
use Newland\Toubiz\Api\ObjectAdapter\ArticleAdapterInterface;
use Newland\Toubiz\Api\ObjectAdapter\Concern\ArticleConstants;
use Newland\Toubiz\Api\ObjectAdapter\Attributes\LodgingAttributes;
use Newland\Toubiz\Api\Service\LanguageAware;
use Newland\Toubiz\Api\Service\Toubiz\Legacy\ObjectAdapter\AbstractLegacyObjectAdapter;
use Newland\Toubiz\Api\Service\UsesFirstMediumAsMainMedium;
use Newland\Toubiz\Api\Service\WithCacheProperty;
use Newland\Toubiz\Api\Utility\AttributeImportUtility;

/**
 * Lodging adapter.
 *
 * This represents an Article with mapping for the Tportal lodging.
 */
class LodgingAdapter extends AbstractLegacyObjectAdapter implements ArticleAdapterInterface, ArticleWithStarRatings
{
    use LanguageAware,
        WithCacheProperty,
        UsesFirstMediumAsMainMedium,
        StarRatingCommon;

    /** @var string|null */
    private $detailUriTemplate;

    public function setDetailUriTemplate(?string $detailUriTemplate): void
    {
        $this->detailUriTemplate = $detailUriTemplate;
    }

    public function getMainType(): int
    {
        return ArticleConstants::TYPE_LODGING;
    }

    public function getExternalId(): string
    {
        return strip_tags((string) $this->object['housedata']['out']['ID']);
    }

    public function getName(): string
    {
        $name = $this->object['housedata']['out']['sortName'];
        if (empty($name)) {
            $name = $this->object['housedata']['out']['address']['personFirstName'];
            $name .= ' ';
            $name .= $this->object['housedata']['out']['address']['personLastName'];
        }
        if (empty(trim($name))) {
            $name = $this->object['housedata']['out']['describingName'];
        }

        $name = strip_tags($name);

        // The name may end with a comma or a dash.
        $suffix = $name[strlen($name) - 1];
        if ($suffix === ',' || $suffix === '-') {
            $name = substr($name, 0, -1);
        }

        return $name;
    }

    public function getLatitude(): ?float
    {
        return $this->object['housedata']['out']['address']['gmapsLatitude'] ?? null;
    }

    public function getLongitude(): ?float
    {
        return $this->object['housedata']['out']['address']['gmapsLongitude'] ?? null;
    }

    public function getMainAddress(): AddressAdapterInterface
    {
        $address = $this->object;
        $address['name'] = $this->getName();
        return new LodgingAddressAdapter($address);
    }

    public function getAbstract(): ?string
    {
        return null;
    }

    public function getDescription(): ?string
    {
        $part = $this->object['housedata']['out']['descriptions'];
        if ($part) {
            if (array_key_exists('description', $part)) {
                return $part['description'];
            }

            $descriptionPart = $this->findInArray(
                $part,
                'descriptionTypeName',
                'Beschreibung'
            );

            if ($descriptionPart && !empty($descriptionPart['description'])) {
                return $descriptionPart['description'];
            }
        }

        return null;
    }

    /**
     * @return CategoryAdapter[]
     */
    public function getCategories(): array
    {
        if (empty($this->object['housedata']['out']['touristicType'])) {
            return [];
        }

        return [ new CategoryAdapter($this->object['housedata']['out']['touristicType']) ];
    }

    public function getAddresses(): array
    {
        return [];
    }

    /**
     * @return MediumAdapter[]
     */
    public function getMedia(): array
    {
        return $this->cache(
            'media',
            function () {
                if (!empty($this->object['housedata']['out']['multimediaURLs'])) {
                    foreach ($this->object['housedata']['out']['multimediaURLs'] as $item) {
                        /*
                         * Right now, only the preview image is being imported as there is
                         * no show view for lodgings.
                         *
                         * Next to that, $item can also be a non-array wit useless
                         * information (like a string), or an array with an unexpected
                         * structure that does not provide a complete image.
                         */
                        if (is_array($item) && ($item['type']['value'] ?? null) === 'Preview') {
                            return [ new MediumAdapter($item) ];
                        }
                    }
                }

                /*
                 * And for the case where there is no 'TP_Preview' despite that every
                 * record basically must have one, there is 'previewImageURL' but with
                 * an already cropped image.
                 */
                $url = $this->object['searchresult']['detailInfo']['previewImageURL'] ?? '';
                if (!empty($item['url'])) {
                    $item = [
                        'ID' => md5($url),
                        'url' => $url,
                    ];
                    return [ new MediumAdapter($item) ];
                }

                return [];
            }
        );
    }

    public function hasAttributes(): bool
    {
        return !empty($this->parseAttributes());
    }

    public function getAttributes(): array
    {
        return AttributeImportUtility::splitAttributeArrayIntoImportables(
            $this->parseAttributes(),
            function ($name, $value) {
                return new AttributeAdapter($this->getExternalId(), $name, $value);
            }
        );
    }

    public function getFiles(): array
    {
        return [];
    }

    public function getSourceName(): ?string
    {
        return null;
    }

    public function getAuthorName(): ?string
    {
        return null;
    }

    public function getBookingUri(): ?string
    {
        return '';
    }

    public function getDetailUri(): ?string
    {
        if (empty($this->object['housedata']['out']['niceUrl'])) {
            return null;
        }

        // Legacy fallback for previous URL handling.
        // TODO Remove together with legacy handling in ArticleUrlService
        if (!$this->detailUriTemplate) {
            return $this->object['housedata']['out']['niceUrl'];
        }

        $niceUrl = ltrim($this->object['housedata']['out']['niceUrl'], '/');
        return str_replace('{tportalUriSegment}', $niceUrl, $this->detailUriTemplate);
    }

    public function getOpeningTimes(): ?string
    {
        return null;
    }

    public function getOpeningTimesFormat(): ?string
    {
        return null;
    }

    public function getAverageRating(): ?int
    {
        $rating = $this->object['searchresult']['serviceProviderRatingTotal']['averageGrade'] ?? null;
        if ($rating !== null) {
            return (int) $rating * 10;
        }
        return null;
    }

    public function getNumberOfRatings(): ?int
    {
        $numberOfRatings = $this->object['searchresult']['serviceProviderRatingTotal']['numberOfRatings'] ?? null;
        if ($numberOfRatings !== null) {
            return (int) $numberOfRatings;
        }
        return null;
    }

    private function parseAttributes(): array
    {
        $attributes = [];

        if (!empty($this->object['searchresult_details']['services'])) {
            $currency = $this->object['searchresult_details']['services'][0]['minimumBasePrice']['currency'];
            $unit = $this->object['searchresult_details']['services'][0]['minimumBasePriceCalculationRule'];
            $value = $this->object['searchresult_details']['services'][0]['minimumBasePrice']['value'];

            if ($currency && $unit && $value) {
                $attributes[LodgingAttributes::MINIMUM_BASE_PRICE] = [
                    LodgingAttributes::PRICE_CURRENCY => $currency,
                    LodgingAttributes::PRICE_UNIT => $unit,
                    LodgingAttributes::PRICE_VALUE => $value,
                ];
            }
        }
        return $attributes;
    }
}
